//================================================================
//
//	Smartronix Proprietary Information
//
//
//	File Name:	sensor_board_model.v
//	Author:	R. Jolly
//	Date:		February 25, 2009
//	Revision:	A
//
//
//	Development:	Environment Control Module UAV
//	Application:	Altera Stratix II
//	Description:	This file contains Verilog code that emulates the Analog Devices AD7934.
//				
//				
//
//
//
//	Revision History:
//
//	DATE		VERSION		DETAILS
//	02/25/09		A		Created by R Jolly
//
//
//
//
//
//
//
//
//
//
//
//==================================================================

module adc_model(

		CLK,
		RESET,

		AD_DATA,
		AD_BUSY,  
		AD_REFOUT,

		AD_CSL,
		AD_RDL,
		AD_CONVST,

        ADC_CONVERSION_REG,
		ADC_SETUP_REG,
		ADC_AVERAGING_REG,
		ADC_RESET_REG

		);

//==================================================================
// Parameter Declarations
//==================================================================
    parameter   IDLE                    = 0,
                CONV_START              = 1,
                SET_BUSY_FLAG           = 2,
                WRITE_ADC_WORD          = 3,
                WRITE_ADC_WORD_DELAY    = 4,
                CS_DE_ASSERTED          = 5;

//==================================================================
// IO Port Declarations
//==================================================================
	// Clock 
    input                       CLK;

	// Reset
	input                       RESET; 

	// AD Stuff
	output  [11:0]              AD_DATA;
	output                      AD_BUSY;
	input                       AD_CSL;
	input                       AD_RDL;
	input                       AD_CONVST;
	output                      AD_REFOUT;
	
	input   [7:0]               ADC_CONVERSION_REG;
	input   [7:0]               ADC_SETUP_REG;
	input   [7:0]               ADC_AVERAGING_REG;
	input   [7:0]               ADC_RESET_REG;



//==================================================================
// Register Declarations
//==================================================================
	wire   [15:0]               RECEIVE_REGISTER;

	reg    [11:0]               AD_DATA;
	reg                         AD_BUSY;
  
  
	// State Machine Registers
	reg	[7:0]					state;
	reg	[7:0]					next;

	// ADC data registers Sensor board
	reg   [15:0]                adc_data_generator;	
	reg                         adc_word_enable;

	// Busy flag counter
	reg	[15:0]                  busy_flag_count;

	// Clock signals
	wire                        CLK;

	// Byte enables
	wire  [5:0]                 byte_enables;
	
	//Channel Data
	reg   [11:0]                channel_1_data;
	reg   [11:0]                channel_2_data;
	reg   [11:0]                channel_3_data;
	reg   [11:0]                channel_4_data;
	
	//Number of Channels register
	reg   [2:0]                 number_of_scan_channels;
	reg                         num_chans_decrement_reg;
	



//==================================================================
// Simulation Directives
//==================================================================

`ifdef SIM
	reg [8*30:1] state_name;
`endif

//=================================================================================
// Assignments	
//=================================================================================

	//assign	AD_DATA = ( !AD_CSL & !AD_RDL ) ? adc_data_generator[11:0]: 12'hzzz;


//=================================================================================
// ADC Cycle: Set the channel data 
//=================================================================================

   always @( negedge CLK or negedge RESET )
   begin
      if ( !RESET )
      begin
        AD_DATA <= 12'h0;
		channel_1_data <= 12'h1ff;
		channel_2_data <= 12'h0c1;
		channel_3_data <= 12'h3f0;
		channel_4_data <= 12'h022;
      end
      else
      begin
         if(state[WRITE_ADC_WORD])
         begin
		    case(number_of_scan_channels)
			3'h1:
               AD_DATA <= channel_1_data;
			3'h2:
               AD_DATA <= channel_2_data;
			3'h3:
               AD_DATA <= channel_3_data;
			3'h4:
               AD_DATA <= channel_4_data;
			default:
               AD_DATA <= channel_1_data;
			endcase
         end
         else if( state[WRITE_ADC_WORD_DELAY] )
         begin
		    channel_1_data <= channel_1_data - adc_data_generator[11:0];
		    channel_2_data <= channel_2_data + adc_data_generator[11:0];
		    channel_3_data <= channel_3_data - adc_data_generator[11:0];
		    channel_4_data <= channel_4_data + adc_data_generator[11:0];
         end
				
      end
   end

	
//=================================================================================
// ADC Cycle: Set Number of channels to scan 
//=================================================================================

   always @( negedge CLK or negedge RESET )
   begin
      if ( !RESET )
      begin
         number_of_scan_channels <= 3'h0;
		 num_chans_decrement_reg <= 1'b0;
      end
      else
      begin
         if(state[CONV_START])
         begin
		    case(ADC_CONVERSION_REG)
			8'h00:
               number_of_scan_channels <= 3'h1;
			8'h08:
               number_of_scan_channels <= 3'h2;
			8'h10:
               number_of_scan_channels <= 3'h3;
			8'h18:
               number_of_scan_channels <= 3'h4;
			default:
               number_of_scan_channels <= 3'h1;
			endcase
         end
         else if( state[WRITE_ADC_WORD_DELAY] & !num_chans_decrement_reg)
         begin
            number_of_scan_channels <= number_of_scan_channels - 1'd1;
			num_chans_decrement_reg <= 1'b1;
         end
         else if( state[CS_DE_ASSERTED] & num_chans_decrement_reg)
         begin
			num_chans_decrement_reg <= 1'b0;
         end
      end
   end


//=================================================================================
// ADC Cycle: busy_flag_count 
//=================================================================================

	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
			busy_flag_count <= 16'h0;
		end
		else
		begin
        		if ( state[SET_BUSY_FLAG] )
			begin
				if( busy_flag_count < 16'hffff )
            			busy_flag_count <= busy_flag_count + 1;
			end
			else if ( state[WRITE_ADC_WORD] )
			begin
				busy_flag_count <= 8'h0;
			end
				
    		end
  end



//=================================================================================
// ADC Cycle: BUSY output register
//=================================================================================

	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
			AD_BUSY <= 1'b0;
		end
		else
		begin
        		if ( state[SET_BUSY_FLAG] )
			begin
					AD_BUSY <= 1'b1;
			end
			else if ( state[WRITE_ADC_WORD] )
			begin
 				AD_BUSY <= 1'b0;
			end
				
    		end
  end

//=================================================================================
//  ADC Cycle:   
//=================================================================================

	always @( negedge CLK or negedge RESET )
	begin
		if ( !RESET )
		begin
      		adc_data_generator <= 16'h1;
        end
/*  		else
    		begin
        		if ( state[SET_BUSY_FLAG] )
          		begin
              		adc_data_generator <= adc_data_generator + 1;
          		end
       		    else if ( state[WRITE_ADC_WORD] )
            		adc_data_generator <= adc_data_generator;
            end
*/	end




//=================================================================================
//	Sensor Board Model Finite State Machine
//	parameter	IDLE					= 0,
///			CONV_START				= 1,
//			SET_BUSY_FLAG			= 2,
//			WRITE_ADC_WORD			= 3,
//			CS_DE_ASSERTED			= 4;
//
//=================================================================================

	always @(  posedge CLK or negedge RESET  )
	begin
		if( !RESET )
		begin
			state <= 8'h00;
			state [IDLE] <= 1'b1;
		end
		else
			state <= next;
	end


	always @( state or AD_CONVST or busy_flag_count or adc_word_enable or 
          		AD_CSL or AD_RDL )
	begin
		next = 8'h00;
		if ( state[IDLE] )
		begin
			if (AD_CONVST)
				next[CONV_START] = 1'b1;

			else
				next[IDLE] = 1'b1;
		end

		if ( state[CONV_START] )
		begin
				next[SET_BUSY_FLAG] = 1'b1;
		end

		if ( state[SET_BUSY_FLAG] )
		begin
			if ( busy_flag_count >= 16'h01ff )
				next[WRITE_ADC_WORD] = 1'b1;
			else
				next[SET_BUSY_FLAG] = 1'b1;
		end

		if ( state[WRITE_ADC_WORD] )
		begin
			if ( !AD_CSL & !AD_RDL)
				next[WRITE_ADC_WORD_DELAY] = 1'b1;
			else
				next[WRITE_ADC_WORD] = 1'b1;
		end

		if ( state[WRITE_ADC_WORD_DELAY] )
		begin
			if ( AD_RDL )
				next[CS_DE_ASSERTED] = 1'b1;
			else
				next[WRITE_ADC_WORD_DELAY] = 1'b1;
		end


		if ( state[CS_DE_ASSERTED] )
		begin
			if ( AD_CSL )
			begin
			   if(number_of_scan_channels > 3'h0)
                  next[WRITE_ADC_WORD] = 1'b1;
			   else
                  next[IDLE] = 1'b1;
			end
            else
               next[CS_DE_ASSERTED] = 1'b1;
		end

	end
//=================================================================
// For Simumlation, display the state names
//
//	parameter	IDLE					= 0,
///			CONV_START				= 1,
//			SET_BUSY_FLAG			= 2,
//			WRITE_ADC_WORD			= 3,
//			WRITE_ADC_WORD_DELAY		= 4,
//			CS_DE_ASSERTED			= 5;
//=================================================================

`ifdef SIM
always @( posedge CLK )
begin
	if ( state == (1 << IDLE))
		state_name = "IDLE";		
	if ( state == (1 << CONV_START))
		state_name = "CONV_START";		
	if ( state == (1 << SET_BUSY_FLAG))
		state_name = "SET_BUSY_FLAG";		
	if ( state == (1 << WRITE_ADC_WORD))
		state_name = "WRITE_ADC_WORD";		
	if ( state == (1 << WRITE_ADC_WORD_DELAY))
		state_name = "WRITE_ADC_WORD_DELAY";		
	if ( state == (1 << CS_DE_ASSERTED))
		state_name = "CS_DE_ASSERTED";		
end
`endif		


endmodule

